// ignore_for_file: non_constant_identifier_names, camel_case_types, missing_return, unused_import, unused_local_variable, dead_code, unnecessary_cast
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tencent_live_fluttify/src/android/android.export.g.dart';
import 'package:tencent_live_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';

class LivePusher {
  LivePusher._();

  static Future<LivePusher> create({bool enableTouchFocus}) async {
    return platform(
      android: (pool) async {
        final instance = LivePusher._();

        final config = await com_tencent_rtmp_TXLivePushConfig.create__();
        if (enableTouchFocus != null) {
          await config.setTouchFocus(enableTouchFocus);
        }

        final context = await android_app_Application.get();
        instance._androidPusher = await com_tencent_rtmp_TXLivePusher
            .create__android_content_Context(context);

        await instance._androidPusher.setConfig(config);

        return instance;
      },
      ios: (pool) async {
        final instance = LivePusher._();

        final config = await TXLivePushConfig.create__();
        if (enableTouchFocus != null) {
          await config.set_touchFocus(enableTouchFocus);
        }

        instance._iosPusher = await TXLivePush.create__();

        await instance._iosPusher.initWithConfig(config);
        return instance;
      },
    );
  }

  com_tencent_rtmp_TXLivePusher _androidPusher;
  TXLivePush _iosPusher;

  final _androidEventDelegate = _AndroidEventDelegate();
  final _iosEventDelegate = _IOSEventDelegate();

  /// 开始预览
  Future<void> startPreview(dynamic preview) async {
    assert(
      preview is com_tencent_rtmp_ui_TXCloudVideoView || preview is UIView,
    );
    return platform(
      android: (pool) async {
        await _androidPusher.startCameraPreview(preview);
      },
      ios: (pool) async {
        await _iosPusher.startPreview(preview);
      },
    );
  }

  /// 开始推流
  Future<void> startPush(String pushUrl) async {
    return platform(
      android: (pool) async {
        final result = await _androidPusher.startPusher(pushUrl);
        debugPrint('result: $result');
      },
      ios: (pool) async {
        final result = await _iosPusher.startPush(pushUrl);
        debugPrint('result: $result');
      },
    );
  }

  /// 暂停推流
  Future<void> pausePush() async {
    return platform(
      android: (pool) => _androidPusher.pausePusher(),
      ios: (pool) => _iosPusher.pausePush(),
    );
  }

  /// 停止预览
  Future<void> stopPreview() async {
    return platform(
      android: (pool) => _androidPusher.stopCameraPreview(true),
      ios: (pool) => _iosPusher.stopPreview(),
    );
  }

  /// 停止推流
  Future<void> stopPush() async {
    return platform(
      android: (pool) => _androidPusher.stopPusher(),
      ios: (pool) => _iosPusher.stopPush(),
    );
  }

  /// 切换前后摄像头
  Future<void> switchCamera() async {
    return platform(
      android: (pool) => _androidPusher.switchCamera(),
      ios: (pool) => _iosPusher.switchCamera(),
    );
  }

  /// 打开/关闭闪光灯
  ///
  /// 仅在当前是后置摄像头时有效
  Future<void> turnFlashLight(bool on) async {
    return platform(
      android: (pool) => _androidPusher.turnOnFlashLight(on),
      ios: (pool) => _iosPusher.toggleTorch(on),
    );
  }

  /// 播放背景音乐
  ///
  /// 调用 TXLivePush 中的 BGM 相关接口可以实现背景混音功能。背景混音是指主播在直播时可以选取一首歌曲伴唱，
  /// 歌曲会在主播的手机端播放出来，同时也会被混合到音视频流中被观众端听到，所以被称为“混音”
  Future<void> playBGM(String path) async {
    return platform(
      android: (pool) => _androidPusher.playBGM(path),
      ios: (pool) => _iosPusher.playBGM(path),
    );
  }

  /// 停止背景音乐
  Future<void> stopBGM() async {
    return platform(
      android: (pool) => _androidPusher.stopBGM(),
      ios: (pool) => _iosPusher.stopBGM(),
    );
  }

  /// 暂停背景音乐
  Future<void> pauseBGM() async {
    return platform(
      android: (pool) => _androidPusher.pauseBGM(),
      ios: (pool) => _iosPusher.pauseBGM(),
    );
  }

  /// 恢复背景音乐
  Future<void> resumeBGM() async {
    return platform(
      android: (pool) => _androidPusher.resumeBGM(),
      ios: (pool) => _iosPusher.resumeBGM(),
    );
  }

  /// 麦克风音量大小
  Future<void> setMicVolume(double volume) async {
    return platform(
      android: (pool) => _androidPusher.setMicVolume(volume),
      ios: (pool) => _iosPusher.setMicVolume(volume),
    );
  }

  /// 背景音乐音量大小
  Future<void> setBGMVolume(double volume) async {
    return platform(
      android: (pool) => _androidPusher.setBGMVolume(volume),
      ios: (pool) => _iosPusher.setBGMVolume(volume),
    );
  }

  /// 背景音乐音调高低
  Future<void> setBgmPitch(double level) async {
    return platform(
      android: (pool) => _androidPusher.setBGMPitch(level),
      ios: (pool) => _iosPusher.setBGMPitch(level),
    );
  }

  /// 设置混音效果
  Future<void> setReverbType(ReverbType type) async {
    return platform(
      android: (pool) async {
        int androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_0;
        switch (type) {
          case ReverbType.REVERB_TYPE_0:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_0;
            break;
          case ReverbType.REVERB_TYPE_1:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_1;
            break;
          case ReverbType.REVERB_TYPE_2:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_2;
            break;
          case ReverbType.REVERB_TYPE_3:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_3;
            break;
          case ReverbType.REVERB_TYPE_4:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_4;
            break;
          case ReverbType.REVERB_TYPE_5:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_5;
            break;
          case ReverbType.REVERB_TYPE_6:
            androidType = com_tencent_rtmp_TXLiveConstants.REVERB_TYPE_6;
            break;
          case ReverbType.REVERB_TYPE_7:
            debugPrint('android端暂无此类型混响效果');
            break;
        }
        await _androidPusher.setReverb(androidType);
      },
      ios: (pool) async {
        TXReverbType iosType = TXReverbType.REVERB_TYPE_0;
        switch (type) {
          case ReverbType.REVERB_TYPE_0:
            iosType = TXReverbType.REVERB_TYPE_0;
            break;
          case ReverbType.REVERB_TYPE_1:
            iosType = TXReverbType.REVERB_TYPE_1;
            break;
          case ReverbType.REVERB_TYPE_2:
            iosType = TXReverbType.REVERB_TYPE_2;
            break;
          case ReverbType.REVERB_TYPE_3:
            iosType = TXReverbType.REVERB_TYPE_3;
            break;
          case ReverbType.REVERB_TYPE_4:
            iosType = TXReverbType.REVERB_TYPE_4;
            break;
          case ReverbType.REVERB_TYPE_5:
            iosType = TXReverbType.REVERB_TYPE_5;
            break;
          case ReverbType.REVERB_TYPE_6:
            iosType = TXReverbType.REVERB_TYPE_6;
            break;
          case ReverbType.REVERB_TYPE_7:
            iosType = TXReverbType.REVERB_TYPE_7;
            break;
        }
        await _iosPusher.setReverbType(iosType);
      },
    );
  }

  /// 设置变调效果
  Future<void> setVoiceChangerType(VoiceChangerType type) async {
    return platform(
      android: (pool) async {
        int androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_0;
        switch (type) {
          case VoiceChangerType.VOICECHANGER_TYPE_0:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_0;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_1:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_1;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_2:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_2;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_3:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_3;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_4:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_4;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_5:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_5;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_6:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_6;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_7:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_7;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_8:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_8;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_9:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_9;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_10:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_10;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_11:
            androidType = com_tencent_rtmp_TXLiveConstants.VOICECHANGER_TYPE_11;
            break;
        }
        await _androidPusher.setVoiceChangerType(androidType);
      },
      ios: (pool) async {
        TXVoiceChangerType iosType = TXVoiceChangerType.VOICECHANGER_TYPE_0;
        switch (type) {
          case VoiceChangerType.VOICECHANGER_TYPE_0:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_0;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_1:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_1;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_2:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_2;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_3:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_3;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_4:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_4;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_5:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_5;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_6:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_6;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_7:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_7;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_8:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_8;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_9:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_9;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_10:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_10;
            break;
          case VoiceChangerType.VOICECHANGER_TYPE_11:
            iosType = TXVoiceChangerType.VOICECHANGER_TYPE_11;
            break;
        }
        await _iosPusher.setVoiceChangerType(iosType);
      },
    );
  }

  /// 事件处理
  ///
  /// [onWaringNetBusy]弱网提醒
  Future<void> setOnEventListener({
    VoidCallback onWaringNetBusy,
    VoidCallback onWaringReconnect,
    VoidCallback onWaringHardwareAccelerationFail,
    VoidCallback onWaringDNSFail,
    VoidCallback onWaringServerConnFail,
    VoidCallback onWaringShakeFail,
    VoidCallback onWaringServerDisconnect,
    VoidCallback onEventConnectSucc,
    VoidCallback onEventPushBegin,
    VoidCallback onEventOpenCameraSuccess,
    VoidCallback onErrorOpenCameraFail,
    VoidCallback onErrorOpenMicFail,
    VoidCallback onErrorVideoEncodeFail,
    VoidCallback onErrorAudioEncodeFail,
    VoidCallback onErrorUnsupportedResolution,
    VoidCallback onErrorUnsupportedSampleRate,
    VoidCallback onErrorNetDisconnect,
  }) async {
    return platform(
      android: (pool) async {
        _androidPusher.setPushListener(
          _androidEventDelegate
            .._onWaringNetBusy = onWaringNetBusy
            .._onWaringReconnect = onWaringReconnect
            .._onWaringHardwareAccelerationFail =
                onWaringHardwareAccelerationFail
            .._onWaringDNSFail = onWaringDNSFail
            .._onWaringServerConnFail = onWaringServerConnFail
            .._onWaringShakeFail = onWaringShakeFail
            .._onWaringServerDisconnect = onWaringServerDisconnect
            .._onEventConnectSucc = onEventConnectSucc
            .._onEventPushBegin = onEventPushBegin
            .._onEventOpenCameraSuccess = onEventOpenCameraSuccess
            .._onErrorOpenCameraFail = onErrorOpenCameraFail
            .._onErrorOpenMicFail = onErrorOpenMicFail
            .._onErrorVideoEncodeFail = onErrorVideoEncodeFail
            .._onErrorAudioEncodeFail = onErrorAudioEncodeFail
            .._onErrorUnsupportedResolution = onErrorUnsupportedResolution
            .._onErrorUnsupportedSampleRate = onErrorUnsupportedSampleRate
            .._onErrorNetDisconnect = onErrorNetDisconnect,
        );
      },
      ios: (pool) async {
        _iosPusher.set_delegate(
          _iosEventDelegate
            .._onWaringNetBusy = onWaringNetBusy
            .._onWaringReconnect = onWaringReconnect
            .._onWaringHardwareAccelerationFail =
                onWaringHardwareAccelerationFail
            .._onWaringDNSFail = onWaringDNSFail
            .._onWaringServerConnFail = onWaringServerConnFail
            .._onWaringShakeFail = onWaringShakeFail
            .._onWaringServerDisconnect = onWaringServerDisconnect
            .._onEventConnectSucc = onEventConnectSucc
            .._onEventPushBegin = onEventPushBegin
            .._onEventOpenCameraSuccess = onEventOpenCameraSuccess
            .._onErrorOpenCameraFail = onErrorOpenCameraFail
            .._onErrorOpenMicFail = onErrorOpenMicFail
            .._onErrorVideoEncodeFail = onErrorVideoEncodeFail
            .._onErrorAudioEncodeFail = onErrorAudioEncodeFail
            .._onErrorUnsupportedResolution = onErrorUnsupportedResolution
            .._onErrorUnsupportedSampleRate = onErrorUnsupportedSampleRate
            .._onErrorNetDisconnect = onErrorNetDisconnect,
        );
      },
    );
  }
}

class _AndroidEventDelegate extends java_lang_Object
    with com_tencent_rtmp_ITXLivePushListener {
  VoidCallback _onWaringNetBusy;
  VoidCallback _onWaringReconnect;
  VoidCallback _onWaringHardwareAccelerationFail;
  VoidCallback _onWaringDNSFail;
  VoidCallback _onWaringServerConnFail;
  VoidCallback _onWaringShakeFail;
  VoidCallback _onWaringServerDisconnect;

  VoidCallback _onEventConnectSucc;
  VoidCallback _onEventPushBegin;
  VoidCallback _onEventOpenCameraSuccess;

  VoidCallback _onErrorOpenCameraFail;
  VoidCallback _onErrorOpenMicFail;
  VoidCallback _onErrorVideoEncodeFail;
  VoidCallback _onErrorAudioEncodeFail;
  VoidCallback _onErrorUnsupportedResolution;
  VoidCallback _onErrorUnsupportedSampleRate;
  VoidCallback _onErrorNetDisconnect;

  @override
  Future<void> onPushEvent(int var1, android_os_Bundle var2) async {
    super.onPushEvent(var1, var2);
    // 网络环境不佳 1101
    if (var1 == 1101 && _onWaringNetBusy != null) {
      _onWaringNetBusy();
    }
    // 网络断连，已启动自动重连（自动重连连续失败超过三次会放弃） 1102
    else if (var1 == 1102 && _onWaringReconnect != null) {
      _onWaringReconnect();
    }
    // 硬编码启动失败，采用软编码 1103
    else if (var1 == 1103 && _onWaringHardwareAccelerationFail != null) {
      _onWaringHardwareAccelerationFail();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (var1 == 3001 && _onWaringDNSFail != null) {
      _onWaringDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (var1 == 3002 && _onWaringServerConnFail != null) {
      _onWaringServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (var1 == 3003 && _onWaringShakeFail != null) {
      _onWaringShakeFail();
    }
    // RTMP 服务器主动断开连接（会触发重试流程） 3004
    else if (var1 == 3004 && _onWaringServerDisconnect != null) {
      _onWaringServerDisconnect();
    }
    // 已经成功连接到腾讯云推流服务器 1001
    else if (var1 == 1001 && _onEventConnectSucc != null) {
      _onEventConnectSucc();
    }
    // 与服务器握手完毕，一切正常，准备开始推流 1002
    else if (var1 == 1002 && _onEventPushBegin != null) {
      _onEventPushBegin();
    }
    // 推流器已成功打开摄像头（部分 Android 手机在此过程需要耗时1s - 2s） 1003
    else if (var1 == 1003 && _onEventOpenCameraSuccess != null) {
      _onEventOpenCameraSuccess();
    }
    // 打开摄像头失败 -1301
    else if (var1 == -1301 && _onErrorOpenCameraFail != null) {
      _onErrorOpenCameraFail();
    }
    // 打开麦克风失败 -1302
    else if (var1 == -1302 && _onErrorOpenMicFail != null) {
      _onErrorOpenMicFail();
    }
    // 视频编码失败 -1303
    else if (var1 == -1303 && _onErrorVideoEncodeFail != null) {
      _onErrorVideoEncodeFail();
    }
    // 音频编码失败 -1304
    else if (var1 == -1304 && _onErrorAudioEncodeFail != null) {
      _onErrorAudioEncodeFail();
    }
    // 不支持的视频分辨率 -1305
    else if (var1 == -1305 && _onErrorUnsupportedResolution != null) {
      _onErrorUnsupportedResolution();
    }
    // 不支持的音频采样率 -1306
    else if (var1 == -1306 && _onErrorUnsupportedSampleRate != null) {
      _onErrorUnsupportedSampleRate();
    }
    // 网络断连，且经三次重连无效，更多重试请自行重启推流 -1307
    else if (var1 == -1306 && _onErrorNetDisconnect != null) {
      _onErrorNetDisconnect();
    }
  }

  @override
  Future<void> onNetStatus(android_os_Bundle var1) async {
    super.onNetStatus(var1);
  }
}

class _IOSEventDelegate extends NSObject with TXLivePushListener {
  VoidCallback _onWaringNetBusy;
  VoidCallback _onWaringReconnect;
  VoidCallback _onWaringHardwareAccelerationFail;
  VoidCallback _onWaringDNSFail;
  VoidCallback _onWaringServerConnFail;
  VoidCallback _onWaringShakeFail;
  VoidCallback _onWaringServerDisconnect;

  VoidCallback _onEventConnectSucc;
  VoidCallback _onEventPushBegin;
  VoidCallback _onEventOpenCameraSuccess;

  VoidCallback _onErrorOpenCameraFail;
  VoidCallback _onErrorOpenMicFail;
  VoidCallback _onErrorVideoEncodeFail;
  VoidCallback _onErrorAudioEncodeFail;
  VoidCallback _onErrorUnsupportedResolution;
  VoidCallback _onErrorUnsupportedSampleRate;
  VoidCallback _onErrorNetDisconnect;

  @override
  Future<void> onPushEvent_withParam(int EvtID, Map param) async {
    super.onPushEvent_withParam(EvtID, param);
    // 网络环境不佳 1101
    if (EvtID == 1101 && _onWaringNetBusy != null) {
      _onWaringNetBusy();
    }
    // 网络断连，已启动自动重连（自动重连连续失败超过三次会放弃） 1102
    else if (EvtID == 1102 && _onWaringReconnect != null) {
      _onWaringReconnect();
    }
    // 硬编码启动失败，采用软编码 1103
    else if (EvtID == 1103 && _onWaringHardwareAccelerationFail != null) {
      _onWaringHardwareAccelerationFail();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (EvtID == 3001 && _onWaringDNSFail != null) {
      _onWaringDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (EvtID == 3002 && _onWaringServerConnFail != null) {
      _onWaringServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (EvtID == 3003 && _onWaringShakeFail != null) {
      _onWaringShakeFail();
    }
    // RTMP 服务器主动断开连接（会触发重试流程） 3004
    else if (EvtID == 3004 && _onWaringServerDisconnect != null) {
      _onWaringServerDisconnect();
    }
    // 已经成功连接到腾讯云推流服务器 1001
    else if (EvtID == 1001 && _onEventConnectSucc != null) {
      _onEventConnectSucc();
    }
    // 与服务器握手完毕，一切正常，准备开始推流 1002
    else if (EvtID == 1002 && _onEventPushBegin != null) {
      _onEventPushBegin();
    }
    // 推流器已成功打开摄像头（部分 Android 手机在此过程需要耗时1s - 2s） 1003
    else if (EvtID == 1003 && _onEventOpenCameraSuccess != null) {
      _onEventOpenCameraSuccess();
    }
    // 打开摄像头失败 -1301
    else if (EvtID == -1301 && _onErrorOpenCameraFail != null) {
      _onErrorOpenCameraFail();
    }
    // 打开麦克风失败 -1302
    else if (EvtID == -1302 && _onErrorOpenMicFail != null) {
      _onErrorOpenMicFail();
    }
    // 视频编码失败 -1303
    else if (EvtID == -1303 && _onErrorVideoEncodeFail != null) {
      _onErrorVideoEncodeFail();
    }
    // 音频编码失败 -1304
    else if (EvtID == -1304 && _onErrorAudioEncodeFail != null) {
      _onErrorAudioEncodeFail();
    }
    // 不支持的视频分辨率 -1305
    else if (EvtID == -1305 && _onErrorUnsupportedResolution != null) {
      _onErrorUnsupportedResolution();
    }
    // 不支持的音频采样率 -1306
    else if (EvtID == -1306 && _onErrorUnsupportedSampleRate != null) {
      _onErrorUnsupportedSampleRate();
    }
    // 网络断连，且经三次重连无效，更多重试请自行重启推流 -1307
    else if (EvtID == -1306 && _onErrorNetDisconnect != null) {
      _onErrorNetDisconnect();
    }
  }

  @override
  Future<void> onNetStatus(Map param) async {
    super.onNetStatus(param);
  }
}
